@@ -6,215 +6,15 @@ use std::sync::Mutex;
 #[cfg(feature = "latest-gtk")]
 use gdk4::prelude::ActionExt;
 
-#[cfg(feature = "widgets")]
-use glib::ObjectExt;
-#[cfg(feature = "widgets")]
-use libadwaita as adw;
-
 use crate::print_perf;
 
 mod action;
 pub use action::*;
 
-struct Selector<S: std::fmt::Debug + Clone + Default + 'static> {
-    name: String,
-    full: bool,
-    selector: Box<dyn Fn(&S) -> S + 'static>,
-    last_state: S,
-    callback: Arc<dyn Fn(S) + 'static>,
-}
-
-impl<S: std::fmt::Debug + Clone + Default + 'static> Selector<S> {
-    fn new(
-        name: &str,
-        full: bool,
-        selector: impl Fn(&S) -> S + 'static,
-        last_state: S,
-        callback: impl Fn(S) + 'static,
-    ) -> Self {
-        Selector {
-            name: name.to_string(),
-            full,
-            selector: Box::new(selector),
-            last_state,
-            callback: Arc::new(callback),
-        }
-    }
-}
-
-impl<S: std::fmt::Debug + Clone + Default + 'static> std::fmt::Debug for Selector<S> {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        f.debug_struct("Selector")
-            .field("name", &self.name)
-            .finish()
-    }
-}
-
-/// A Middleware allows to handle actions before and after state mutation.
-///
-/// This allows implementing side effects for actions.
-pub trait Middleware<S: std::fmt::Debug + Clone + Default + PartialEq + Eq + 'static> {
-    fn pre_reduce(&self, _: &Action, _: &S) {}
-    fn post_reduce(&self, _: &Action, _: &S) {}
-}
-
-/// Sending Actions from GTK widgets via dispatch() performs some checks for
-/// prohibiting internal update loops.
 #[cfg(feature = "widgets")]
-pub trait DispatchWidget: glib::IsA<gtk::Widget> {
-    fn dispatch(&self, action: &str, argument: Option<&dyn glib::ToVariant>) {
-        crate::check_internal!(self);
-        gtk::prelude::WidgetExt::activate_action(
-            self,
-            &format!("app.{}", action),
-            argument.map(|a| a.to_variant()).as_ref(),
-        );
-    }
-}
-
+mod widgets;
 #[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::AppChooser {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::AppChooserButton {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::ApplicationWindow {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::Box {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::Button {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::Calendar {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::CenterBox {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::ColorButton {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::ComboBox {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::Dialog {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::DragIcon {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::DrawingArea {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::DropDown {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::Editable {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::EmojiChooser {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::Entry {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::Expander {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::Fixed {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::FlowBox {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::FontButton {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::Frame {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::Grid {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::GridView {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::HeaderBar {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::IconView {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::Image {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::InfoBar {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::Label {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::LevelBar {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::LinkButton {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::ListBox {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::ListBoxRow {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::LockButton {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::MenuButton {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::MessageDialog {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::Notebook {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::PasswordEntry {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::Popover {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::PopoverMenu {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::Revealer {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::ScrolledWindow {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::SearchBar {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::SearchEntry {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::SpinButton {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::Spinner {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::Stack {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::StackSidebar {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::StackSwitcher {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::Statusbar {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::Switch {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::Text {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::ToggleButton {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::Widget {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for gtk::Window {}
-
-#[cfg(feature = "widgets")]
-impl DispatchWidget for adw::ActionRow {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for adw::Carousel {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for adw::Clamp {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for adw::ComboRow {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for adw::ExpanderRow {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for adw::Flap {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for adw::HeaderBar {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for adw::Leaflet {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for adw::PreferencesGroup {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for adw::PreferencesRow {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for adw::SplitButton {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for adw::Squeezer {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for adw::Swipeable {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for adw::TabBar {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for adw::ViewStack {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for adw::ViewSwitcher {}
-#[cfg(feature = "widgets")]
-impl DispatchWidget for adw::ViewSwitcherBar {}
+pub use widgets::*;
 
 /// A global Store
 ///
@@ -227,7 +27,7 @@ impl DispatchWidget for adw::ViewSwitcherBar {}
 ///
 /// The reducer must not have any side effects (E.g. saving the state to a file).
 ///
-/// Sideeffects are implemented in middlewares (See [Middleware](crate::Middleware)).
+/// Side effects are implemented in middlewares (See [Middleware](crate::Middleware)).
 pub struct Store<S: std::fmt::Debug + Clone + Default + PartialEq + Eq + 'static> {
     state: Arc<Mutex<S>>,
     reducer: Arc<Mutex<Box<dyn Fn(&Action, &mut S) + 'static>>>,
@@ -246,13 +46,17 @@ impl<S: std::fmt::Debug + Clone + Default + PartialEq + Eq + 'static> std::fmt::
 
 impl<S: std::fmt::Debug + Clone + Default + PartialEq + Eq> Store<S> {
     /// Create a new Store.
+    /// # Arguments
+    /// - default_state: The initial state of the store.
+    /// - reducer: The root reducer for this store.
+    /// - middlewares: A list of [Middleware](crate::Middleware).
     pub fn new(
+        default_state: S,
         reducer: impl Fn(&Action, &mut S) + 'static,
         middlewares: Vec<Box<dyn Middleware<S>>>,
     ) -> Self {
-        let state = Default::default();
         Store {
-            state: Arc::new(Mutex::new(state)),
+            state: Arc::new(Mutex::new(default_state)),
             reducer: Arc::new(Mutex::new(Box::new(reducer))),
             selectors: Arc::new(Mutex::new(Default::default())),
             middlewares: Arc::new(Mutex::new(Arc::new(middlewares))),
@@ -436,6 +240,48 @@ impl<S: std::fmt::Debug + Clone + Default + PartialEq + Eq> Store<S> {
     }
 }
 
+struct Selector<S: std::fmt::Debug + Clone + Default + 'static> {
+    name: String,
+    full: bool,
+    selector: Box<dyn Fn(&S) -> S + 'static>,
+    last_state: S,
+    callback: Arc<dyn Fn(S) + 'static>,
+}
+
+impl<S: std::fmt::Debug + Clone + Default + 'static> Selector<S> {
+    fn new(
+        name: &str,
+        full: bool,
+        selector: impl Fn(&S) -> S + 'static,
+        last_state: S,
+        callback: impl Fn(S) + 'static,
+    ) -> Self {
+        Selector {
+            name: name.to_string(),
+            full,
+            selector: Box::new(selector),
+            last_state,
+            callback: Arc::new(callback),
+        }
+    }
+}
+
+impl<S: std::fmt::Debug + Clone + Default + 'static> std::fmt::Debug for Selector<S> {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        f.debug_struct("Selector")
+            .field("name", &self.name)
+            .finish()
+    }
+}
+
+/// A Middleware allows to handle actions before and after state mutation.
+///
+/// This allows implementing side effects for actions.
+pub trait Middleware<S: std::fmt::Debug + Clone + Default + PartialEq + Eq + 'static> {
+    fn pre_reduce(&self, _: &Action, _: &S) {}
+    fn post_reduce(&self, _: &Action, _: &S) {}
+}
+
 #[macro_export]
 macro_rules! dispatch {
     (
@@ -522,15 +368,16 @@ macro_rules! store {
         /// Initialize the store and global state
         ///
         /// # Arguments
-        /// - initializer: Initialize the state.
+        /// - default_state: The initial state.
         /// - reducer: Handle actions and mutate the state
         /// - middlewares: Pre- and post-reduce handlers
         pub fn init_store(
+            default_state: $state,
             reducer: impl Fn(&gstore::Action, &mut $state) + 'static,
             middlewares: Vec<Box<dyn gstore::Middleware<$state>>>,
         ) {
             unsafe {
-                STORE = Some(Store::new(reducer, middlewares));
+                STORE = Some(Store::new(default_state, reducer, middlewares));
                 STORE.as_ref().unwrap().dispatch(gstore::INIT.into(), None);
             }
         }
